category(books).
category(fiction).
category(sci_fi).
category(polar).
category(romance).
category(nonfiction).
category(report).
category(biography).

category(technology).
category(cars).
category(computers).

category(science).
category(inf).
category(math).
category(ai).
category(neurology).

% subcategory(X,Y) -- Y is a subcategory of X 
%                     X is parent of Y in the ontology tree
subcategory(books,fiction).
subcategory(books,nonfiction).
subcategory(fiction,sci_fi).
subcategory(fiction,polar).
subcategory(fiction,romance).
subcategory(nonfiction,report).
subcategory(nonfiction,biography).

subcategory(technology,cars).
subcategory(technology,computers).

subcategory(science, inf).
subcategory(inf, ai).
subcategory(science, math).
subcategory(science, neurology).


% ancestor(X,Y) X is an ancestor of Y in the ontology tree
ancestor(X,X) :- category(X).
ancestor(X,Y) :- 
    subcategory(X,Y).
ancestor(X,Y) :- 
    subcategory(X,Z),
    ancestor(Z,Y).

% X is not a root of the ontology tree if there is category Y of which
% X is a subcategory.
non_root_category(X) :-
    category(X), category(Y),
    subcategory(Y, X).

% X is root category of Y 
% X is the root of the ontology tree Y belongs to

root_category(X,Y) :-
    category(X), category(Y), 
    ancestor(X,Y), 
    not non_root_category(X).
    
    
person(pierre).
person(mathieu).
person(lukas).

interested(pierre, ai).
interested(mathieu, math).
interested(lukas, scifi).

possibly_common_interests(X,Y) :- 
    person(X),
    person(Y), X != Y,
    interested(X,C1),
    interested(Y,C2),
    root_category(R1,C1),
    root_category(R2,C2),
    R1 = R2.
    
likely_common_interest(X,Y) :-
    interested(X, C1),
    interested(Y, C2),
    ancestor(C1, C2).
    
likely_common_interest(X,Y) :-
    interested(X, C1),
    interested(Y, C2),
    subcategory(C, C1),
    subcategory(C, C2).


